<?php
/**
 * @copyright	Copyright (C) 2012 Daniel Calviño Sánchez
 * @license		GNU Affero General Public License version 3 or later; see LICENSE.txt
 */

require_once JPATH_BASE.'/tests/unit/JoomlaTestCase.php';

require_once JPATH_BASE.'/plugins/system/simplecustomrouter/simplecustomrouter.php';
require_once JPATH_BASE.'/libraries/joomla/event/dispatcher.php';

//Needed by JApplication
require_once JPATH_BASE.'/libraries/joomla/environment/request.php';
require_once JPATH_BASE.'/libraries/joomla/environment/uri.php';

class plgSystemSimpleCustomRouterTest extends JoomlaTestCase {

    protected function setUp() {
        JFactory::$config = null;
        
        //Set a JSession stub, as creating a real JSession fails (see the
        //comments on JSessionStub class)
        JFactory::$session = new JSessionStub();

        JFactory::$application = null;
        //It is always the same JApplication, as the instances are stored in a
        //static variable internal to a method.
        JFactory::getApplication('site', array('session' => false));
        //The same JApplication always has the same router, so the router rules
        //have to be emptied between tests.
        $this->setPrivate(JFactory::getApplication()->getRouter(), '_rules', array());
    }
    
    public function testAttachRulesOnInitialization() {
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array());
        $plugin->onAfterInitialise();

        //Check that the SimpleCustomRouter was added to the rules of the router
        $router = JFactory::getApplication()->getRouter();
        $routerRules = $this->getPrivate($router, '_rules');
        
        $this->assertEquals(1, count($routerRules['build']), 'Build rules count');
        $this->assertEquals('SimpleCustomRouter', get_class($routerRules['build'][0][0]), 'Build rule class');
        $this->assertEquals('build', $routerRules['build'][0][1], 'Build rule method');

        $this->assertEquals(1, count($routerRules['parse']), 'Parse rules count');
        $this->assertEquals('SimpleCustomRouter', get_class($routerRules['parse'][0][0]), 'Parse rule class');
        $this->assertEquals('parse', $routerRules['parse'][0][1], 'Parse rule method');
        
        $this->assertTrue($routerRules['build'][0][0] == $routerRules['parse'][0][0], 'Same object in build and parse rules');
    }

    public function testAttachRulesOnAdminInitialization() {
        //Needed for JURI, which is needed by JAdministrator 
        $_SERVER['HTTP_HOST'] = 'localhost';
        
        JFactory::$application = null;
        JFactory::getApplication('administrator', array('session' => false));
        $this->setPrivate(JFactory::getApplication()->getRouter(), '_rules', array());

        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array());
        $plugin->onAfterInitialise();

        //Check that the SimpleCustomRouter was not added to the rules of the
        //router
        $router = JFactory::getApplication()->getRouter();
        $routerRules = $this->getPrivate($router, '_rules');
        
        //As the rules were emptied there will be no 'build' or 'parse' elements
        $this->assertEquals(0, count($routerRules), 'Rules count');
    }
    
    private function getPrivate($object, $property) {
        $reflectionProperty = new ReflectionProperty(get_class($object), $property);
        $reflectionProperty->setAccessible(true);
        return $reflectionProperty->getValue($object);
    }

    private function setPrivate($object, $property, $value) {
        $reflectionProperty = new ReflectionProperty(get_class($object), $property);
        $reflectionProperty->setAccessible(true);
        $reflectionProperty->setValue($object, $value);
    }
    
    public function testCreateCacheDefaultEnabled() {
        $this->setDefaultCacheConfiguration(true, 'file', '15');
                
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array());
        $plugin->onAfterInitialise();
        
        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();
        
        $this->assertNotNull($simpleCustomRouterCache, 'Cache set');
        $this->assertEquals(JFactory::getCache('simplecustomrouter', ''), $simpleCustomRouterCache, 'Cache same as "simplecustomrouter" cache');
        $this->assertTrue($simpleCustomRouterCache->getCaching(), 'Cache enabled');
        $this->assertEquals('JCacheStorageFile', get_class($simpleCustomRouterCache->_getStorage()), 'Cache storage'); 
        $this->assertEquals('15', $simpleCustomRouterCache->cache->_options['lifetime'], 'Cache lifetime'); 
    }

    public function testCreateCacheDefaultEnabledWithoutHandler() {
        $this->setDefaultCacheConfiguration(true, '', '15');
                
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array());
        $plugin->onAfterInitialise();
        
        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();
        
        $this->assertNull($simpleCustomRouterCache, 'Cache not set');
    }

    public function testCreateCacheDefaultDisabled() {
        $this->setDefaultCacheConfiguration(false, 'file', '15');
                
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array());
        $plugin->onAfterInitialise();
        
        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();
        
        $this->assertNull($simpleCustomRouterCache, 'Cache not set');
    }

    public function testCreateCacheDefaultEnabledWhenCustomDisabled() {
        $this->setDefaultCacheConfiguration(true, 'file', '15');
                
        $params = '{"overrideDefaultConfiguration":false, "enableCache":false}';
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array('params' => $params));
        $plugin->onAfterInitialise();
        
        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();
        
        $this->assertNotNull($simpleCustomRouterCache, 'Cache set');
        $this->assertEquals(JFactory::getCache('simplecustomrouter', ''), $simpleCustomRouterCache, 'Cache same as "simplecustomrouter" cache');
        $this->assertTrue($simpleCustomRouterCache->getCaching(), 'Cache enabled');
        $this->assertEquals('JCacheStorageFile', get_class($simpleCustomRouterCache->_getStorage()), 'Cache storage'); 
        $this->assertEquals('15', $simpleCustomRouterCache->cache->_options['lifetime'], 'Cache lifetime'); 
    }

    public function testCreateCacheDefaultEnabledWhenCustomDisabledWithoutHandler() {
        $this->setDefaultCacheConfiguration(true, '', '15');
                
        $params = '{"overrideDefaultConfiguration":false, "enableCache":false}';
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array('params' => $params));
        $plugin->onAfterInitialise();
        
        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();
        
        $this->assertNull($simpleCustomRouterCache, 'Cache not set');
    }
    
    public function testCreateCacheDefaultDisabledWhenCustomEnabled() {
        $this->setDefaultCacheConfiguration(false, 'file', '15');
                
        $params = '{"overrideDefaultConfiguration":false, "enableCache":true}';
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array('params' => $params));
        $plugin->onAfterInitialise();
        
        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();
        
        $this->assertNull($simpleCustomRouterCache, 'Cache not set');
    }
    
    public function testCreateCacheCustomEnabledWhenDefaultDisabled() {
        $this->setDefaultCacheConfiguration(false, 'file', '15');
        
        $params = '{"overrideDefaultConfiguration":true, "enableCache":true}';
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array('params' => $params));
        $plugin->onAfterInitialise();
        
        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();
        
        $this->assertNotNull($simpleCustomRouterCache, 'Cache set');
        $this->assertEquals(JFactory::getCache('simplecustomrouter', ''), $simpleCustomRouterCache, 'Cache same as "simplecustomrouter" cache');
        $this->assertTrue($simpleCustomRouterCache->getCaching(), 'Cache enabled');
        $this->assertEquals('JCacheStorageFile', get_class($simpleCustomRouterCache->_getStorage()), 'Cache storage'); 
        $this->assertEquals('15', $simpleCustomRouterCache->cache->_options['lifetime'], 'Cache lifetime'); 
    }

    public function testCreateCacheCustomEnabledWhenDefaultDisabledWithoutHandler() {
        $this->setDefaultCacheConfiguration(false, '', '15');
                
        $params = '{"overrideDefaultConfiguration":true, "enableCache":true}';
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array('params' => $params));
        $plugin->onAfterInitialise();
        
        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();
        
        $this->assertNull($simpleCustomRouterCache, 'Cache not set');
    }

    public function testCreateCacheCustomDisabledWhenDefaultEnabled() {
        $this->setDefaultCacheConfiguration(true, 'file', '15');
                
        $params = '{"overrideDefaultConfiguration":true, "enableCache":false}';
        $plugin = new plgSystemSimpleCustomRouter(JDispatcher::getInstance(), array('params' => $params));
        $plugin->onAfterInitialise();
        
        $simpleCustomRouterCache = $this->getSimpleCustomRouterCacheFromJoomlaRouter();
        
        $this->assertNull($simpleCustomRouterCache, 'Cache not set');
    }

    private function setDefaultCacheConfiguration($caching, $handler, $lifetime) {
        //Ensure that no cache instances exist
        JFactory::$cache = null;

        //Set the default cache configuration
        JFactory::getConfig()->set('caching', $caching);
        JFactory::getConfig()->set('cache_handler', $handler);
        JFactory::getConfig()->set('cachetime', $lifetime);
    }
    
    private function getSimpleCustomRouterCacheFromJoomlaRouter() {
        $router = JFactory::getApplication()->getRouter();
        $routerRules = $this->getPrivate($router, '_rules');
        
        $simpleCustomRouter = $routerRules['build'][0][0];
        
        return $this->getPrivate($simpleCustomRouter, 'cache');
    }
}

//When a JSession is created (for example, when creating the JApplication), it
//calls to "session_start()". It fails with 
//"Cannot send session cookie - headers already sent by (output started at /usr/share/pear/PHPUnit/Util/Printer.php:174)"
//As the session is not relevant to test the router cache, a JSession stub with
//an empty "_start()" method (the one that called "session_start()" is used.
class JSessionStub extends JSession {
    protected function _start() {
    }
}
